# Python imports
import os
import subprocess
import collections
import tempfile

# Lib imports
import gi
gi.require_version('Gtk', '3.0')
gi.require_version('GtkSource', '4')
from gi.repository import GObject
from gi.repository import Gtk
from gi.repository import Gdk
from gi.repository import GtkSource

# Application imports



class ControllerData:
    ''' ControllerData contains most of the state of the app at ay given time. It also has some support methods. '''

    def setup_controller_data(self) -> None:
        # Needed hack to get GtkSourceView widget to run from glade file...
        GtkSource.View()

        self.window           = settings_manager.get_main_window()
        self.builder          = None
        self.base_container   = None

        self.setup_builder_and_container()

        self.was_midified_key = False
        self.ctrl_down        = False
        self.shift_down       = False
        self.alt_down         = False


        self.language_id   = 'c'
        self.language_name = 'C'
        self.black_color   = Gdk.color_parse('#000000');
        self.gui_style_ids = [
            'bracket-match', 'bracket-mismatch',
            'current-line', 'cursor', 'line-numbers',
            'secondary-cursor', 'selection',
            'selection-unfocused', 'text'
        ]

        self.styles_list             = self.builder.get_object('styles_list')
        self.languages_list          = self.builder.get_object('languages_list')

        self.languages_dropdown      = self.builder.get_object('languages_dropdown')
        self.styles_treeview         = self.builder.get_object('styles_treeview')

        self.bold_tggle_btn          = self.builder.get_object('bold_tggle_btn')
        self.italic_tggle_btn        = self.builder.get_object('italic_tggle_btn')
        self.underline_tggle_btn     = self.builder.get_object('underline_tggle_btn')
        self.strikethrough_tggle_btn = self.builder.get_object('strikethrough_tggle_btn')
        self.fr_check_btn            = self.builder.get_object('fr_check_btn')
        self.br_check_btn            = self.builder.get_object('br_check_btn')
        self.fr_color_btn            = self.builder.get_object('fr_color_btn')
        self.br_color_btn            = self.builder.get_object('br_color_btn')
        self.reset_btn               = self.builder.get_object('reset_btn')

        self.name_entry              = self.builder.get_object('name_entry')
        self.id_entry                = self.builder.get_object('id_entry')
        self.desc_entry              = self.builder.get_object('desc_entry')
        self.authr_entry             = self.builder.get_object('authr_entry')
        self.sample_lbl              = self.builder.get_object('sample_lbl')
        self.src_buffer              = GtkSource.Buffer(max_undo_levels = 0)
        self.src_preview             = self.builder.get_object('src_preview')

        self.scheme_manager          = GtkSource.StyleSchemeManager()
        self.language_manager        = GtkSource.LanguageManager()
        self.all_styles_dict         = collections.OrderedDict()

        self.language                = self.language_manager.get_language(self.language_id)
        self.current_lang_id         = self.language_id
        self.current_lang_name       = self.language_name
        self.orig_scheme_file        = None

        self.bold_tggl_handler          = self.get_handler_id(self.bold_tggle_btn, 'toggled')
        self.italic_tggl_handler        = self.get_handler_id(self.italic_tggle_btn, 'toggled')
        self.underline_tggl_handler     = self.get_handler_id(self.underline_tggle_btn, 'toggled')
        self.strikethrough_tggl_handler = self.get_handler_id(self.strikethrough_tggle_btn, 'toggled')
        self.foreground_tggl_handler    = self.get_handler_id(self.fr_check_btn, 'toggled')
        self.background_tggl_handler    = self.get_handler_id(self.br_check_btn, 'toggled')

        # watch temp directory to help the sample viewer
        self.scheme_manager.append_search_path( tempfile.gettempdir() )
        self.src_preview.set_buffer(self.src_buffer)
        self.load_initial_data()

    def load_initial_data(self, from_file = None):
        if from_file and os.path.isfile(from_file):
            self.load_scheme(from_file)
        else:
            self.load_scheme('cobalt')

        languages = self.language_manager.get_language_ids()
        languages.sort()
        self.lang_map_name_to_id = {}

        self.languages_list.append(['Default styles'])
        self.lang_map_name_to_id['Default styles'] = 'def'

        for lang in languages:
            lang_name = self.language_manager.get_language(lang).get_name()
            self.lang_map_name_to_id[lang_name] = lang

            if lang_name != 'Defaults':
                self.languages_list.append([lang_name])

        self.styles_treeview_selection = self.styles_treeview.get_selection()
        self.styles_treeview_selection.connect('changed', self.on_style_selected)

        # select the first language
        self.languages_dropdown.set_active(0)

        # select the first style
        model     = self.styles_treeview.get_model()
        tree_iter = model.get_iter_first()
        self.styles_treeview_selection.select_iter(tree_iter)
        self.selected_style_id = model[tree_iter][0]

    def get_handler_id(self, obj, signal_name):
        signal_id, detail = GObject.signal_parse_name(signal_name, obj, True)
        return GObject.signal_handler_find(obj, GObject.SignalMatchType.ID, signal_id, detail, None, None, None)

    def get_base_container(self):
        return self.base_container

    def clear_console(self) -> None:
        ''' Clears the terminal screen. '''
        os.system('cls' if os.name == 'nt' else 'clear')

    def call_method(self, _method_name: str, data: type) -> type:
        '''
        Calls a method from scope of class.

                Parameters:
                        a (obj): self
                        b (str): method name to be called
                        c (*): Data (if any) to be passed to the method.
                                Note: It must be structured according to the given methods requirements.

                Returns:
                        Return data is that which the calling method gives.
        '''
        method_name = str(_method_name)
        method      = getattr(self, method_name, lambda data: f"No valid key passed...\nkey={method_name}\nargs={data}")
        return method(*data) if data else method()

    def has_method(self, obj: type, method: type) -> type:
        ''' Checks if a given method exists. '''
        return callable(getattr(obj, method, None))

    def clear_children(self, widget: type) -> None:
        ''' Clear children of a gtk widget. '''
        for child in widget.get_children():
            widget.remove(child)

    def get_clipboard_data(self, encoding = "utf-8") -> str:
        proc    = subprocess.Popen(['xclip','-selection', 'clipboard', '-o'], stdout = subprocess.PIPE)
        retcode = proc.wait()
        data    = proc.stdout.read()
        return data.decode(encoding).strip()

    def set_clipboard_data(self, data: type, encoding = "utf-8") -> None:
        proc = subprocess.Popen(['xclip','-selection','clipboard'], stdin = subprocess.PIPE)
        proc.stdin.write(data.encode(encoding))
        proc.stdin.close()
        retcode = proc.wait()
